#!/bin/bash

while getopts ":a:co:n:w:" opt; do
  case $opt in
    a) 
    # specify tested architecture
    arch="$OPTARG"
    ;;
    o) 
    # specify the rpm absolute path
    old_rpmdir="$OPTARG"
    ;;
    n)
    # specify the rpm absolute path
    new_rpmdir="$OPTARG"
    ;;
    w) 
    # specify the work space
    workdir="$OPTARG"
    ;;
    \?) 
    echo "Invalid option -$OPTARG" >&2
    exit 1
    ;;
  esac
done

# we need all the argument, so add check on it
if [ -z "$arch" ]; then
  echo "use -a to specify arch"
  exit 1
fi

if [ -z "$old_rpmdir" ]; then
  echo "use -r to specify taskid"
  exit 1
fi

if [ -z "$new_rpmdir" ]; then
  echo "use -r to specify taskid"
  exit 1
fi 

if [ -z "$workdir" ]; then
  echo "use -w to specify work space"
  exit 1
fi
# prepare
sudo dnf install createrepo -y

# we use a simple chroot env to test installation
# maybe need to change to docker
mkdir -p $workdir

cd $workdir/
# hard-code, need change
wget https://git.opencloudos.tech/sources-stream/images/ocs_install_${arch}.sqfs

cd $workdir

sudo unsquashfs -f -d tss ocs_install_${arch}.sqfs
sed -i '$a 124.225.118.191  mirrors.opencloudos.tech' $workdir/tss/etc/hosts

sed -i 's/172.16.0.13/175.178.188.29/g' $workdir/tss/etc/hosts

check_executable(){
# sometime exit 1 doesn't mean failure
    set +e
    success=1
    local result="${1}_result"
    echo "\n========= **Start ${arch} Test.** ========="
    echo "============== ** Start check ${1} executable **=================="
    echo '{}' > ${1}_${arch}_executable_result.json
    while read -r line;
    do

        echo "Install $line: "
        sudo chroot tss dnf install --best $line -y 2>&1
        if [ $? -ne 0 ]; then
            echo "\`Installed\` $line \`failed\`."
            echo "no need to test remove. "
            cat single_install_info >> $result
            echo "===============================\n"
            echo >> $result
            success=0
        else
            grep -q "scriptlet failed" single_install_info
            if [ $? -eq 0 ]; then
                echo "\`Installed\` $line \`success\` with \`scriptlet failed\`."
                err_info=`grep "scriptlet failed" single_install_info`
                echo "${err_info}"
            else
                echo "\`Installed\` $line \`success\`."
                grep -q "warning:" single_install_info
                if [ $? -eq 0 ]; then
                    echo "with some \`warnings: \`"
                fi
            fi
            # 安装一个包，开始check_compatibility
            supported_types=("ASCII text executable" "ELF 64-bit LSB")
            unsupported_types="ELF 64-bit LSB shared object"
            args=("--help" "-h" "-v" "--info")
            output=""
            package=$(sudo chroot tss dnf repoquery --qf "%{NAME}" --installed $line)
            echo "package $package"
            rpm -ql "$package" | while read file; do
                if echo "$file" | grep "\.debug$"; then
                    continue
                fi

                file="$file"
                file_type=$(file -b "$file")
                supported_type=0
                for t in "${supported_types[@]}"; do
                    if echo "$file_type" | grep -w "$t" | grep -E -v "$unsupported_types"; then
                        supported_type=1
                        break
                    fi
                done
                if [ "$supported_type" -eq 1 ]; then
                    echo "========================================command $file========================================" | tee -a "${1}_${arch}_executable_result.log"
                    for arg in "${args[@]}"; do
                        sudo chroot tss $file $arg
                        argList=$(sudo chroot tss $file $arg 2>&1)
                        command_key="$file $arg"
                        if [ $? -ne 0 ]; then
                            echo "**********$command_key failed**********" | tee -a "${1}_${arch}_compat_executable_result.log"
                            jq ". | .[\"$command_key\"] = \"$command_key failed\"" ${1}_${arch}_executable_result.json > tmp && mv tmp ${1}_${arch}_executable_result.json
                            continue  # Skip to the next argument
                        fi
                        jq ". | .[\"$command_key\"] = \"$argList\"" ${1}_${arch}_executable_result.json > tmp && mv tmp ${1}_${arch}_executable_result.json
                        echo "**********$command_key success**********" | tee -a "${1}_${arch}_compat_executable_result.log"
                        echo "$argList" | tee -a "${1}_${arch}_compat_executable_result.log"
                        break
                    done
                fi
            done
        fi
        echo "dnf install --best $line -y" 


        ls $workdir/${1}_pkgdir | grep -q rpm

        if [ $? -ne 0 ]; then
            echo "no $arch rpm need to install"
        fi

        if [ $success -ne 1 ]; then
            echo "check ${1} executable failed"
        fi

    done < ${1}_pkg_list
    echo "\n========= **End $arch $1 install.** ========="
   # echo "=========  test installation done ==========="   
}


# 1. we install the old package
mkdir -p old_pkgdir
cd $workdir/old_pkgdir

cp ${old_rpmdir}/*.${arch}.* $workdir/old_pkgdir/ -a
cp ${old_rpmdir}/*.noarch.* $workdir/old_pkgdir/ -a

rm -f $workdir/old_pkgdir/*debuginfo*
rm -f $workdir/old_pkgdir/*debugsource*
rm -f $workdir/old_pkgdir/*.src.rpm

ls ./*.rpm | xargs -I {} rpm -q --queryformat "%{NAME}\n" {} 2>/dev/null > $workdir/old_pkg_list

cd $workdir
check_executable old

if [ $success -ne 1 ]; then
    cd $workdir
    rm -rf ocs_install_$arch.sqfs 
    rm -rf tss
    rm -rf old_pkgdir
    set -e
    exit 1
fi
# ==================================================================

# 2. we install the new package

mkdir -p new_pkgdir
cd $workdir/new_pkgdir


cp ${new_rpmdir}/*.${arch}.* $workdir/new_pkgdir/ -a
cp ${new_rpmdir}/*.noarch.* $workdir/new_pkgdir/ -a

rm -f $workdir/new_pkgdir/*debuginfo*
rm -f $workdir/new_pkgdir/*debugsource*
rm -f $workdir/new_pkgdir/*.src.rpm

# sometimes, we need regen the chroot, so maintain in new_pkgdir
# use cp here.
cp $workdir/new_pkgdir/* $workdir/tss/data/repo


cd $workdir/tss/data/repo


createrepo -d $workdir/tss/data/repo

# got all the tested rpms
cd $workdir/tss/data/repo

ls ./*.rpm | xargs -I {} rpm -q --queryformat "%{NAME}-%{VERSION}-%{RELEASE}\n" {} 2>/dev/null > $workdir/new_pkg_list

cd $workdir
sudo chroot tss dnf makecache
sudo chroot tss dnf update -y

check_executable new
# =================================================================


cd $workdir
rm -rf ocs_install_$arch.sqfs 
rm -rf tss
rm -rf new_pkgdir old_pkgdir
set -e

if [ $success -ne 1 ]; then
    exit 1
fi
