from ruamel.yaml import YAML
from tqdm import tqdm
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor, as_completed
import threading
import multiprocessing
import requests
import json

token = "XXX"


all_pkg = []
global_pr_count = 0
global_compat_failed = 0


lock = threading.Lock()

check_list = {
    'compat_exec_failed': 0,
    'compat_api_failed': 0,
    'compat_abi_failed': 0,
    'compat_ability_failed': 0, 
    'compat_conf_failed': 0,
    'compat_filelist_failed': 0, 
    'compat_subrpm_failed': 0,
    'AI_disagree_exec': 0 ## 2024.8.15
}

def get_pkg_list(limit):
    file_path = '/data/gitee/rpm_info/packages.yaml'
    
    yaml = YAML()
    
    with open(file_path, 'r') as file:
        data = yaml.load(file)
    
    for item in data:
        if 'sig' in item and item['sig'] == 'default':
            for pkg in item['packages']:
                if limit > 0:
                    all_pkg.append(pkg)
                    limit -= 1

## data example:
##{
##    'pkg_name': None,
##    'pr1' : {
##        'url': 'http:/XXXX',
##        'label': [A, B, C],
##        'merge_time': XXXX-XX-XX
##    }
##    'pr2' : {
##        'url': 'http:/XXXX',
##        'label': [A, B, C],
##        'merge_time': XXXX-XX-XX
##    }
##}
def get_pr_labels(package, since_time):
    global global_pr_count, global_compat_failed, token
    global check_list

    #print(f"Thread {threading.current_thread().name} is processing package {package}")

    # 设置请求头
    headers = {
        'Content-Type': 'application/json;charset=UTF-8'
    }
    
    # 设置URL和参数
    url = f'https://gitee.com/api/v5/repos/opencloudos-stream/{package}/pulls'
    params = {
        'access_token': token,
        'state': 'merged',
        'since': since_time
    }

    pr_count = 0
    compat_failed_count = 0
    
    pr_detail = {}
    pr_detail['pkg_name'] = package

    # 执行GET请求
    response = requests.get(url, headers=headers, params=params)
    
    # 检查请求是否成功
    if response.status_code == 200:
        # 解析返回的JSON数据
        data = json.loads(response.text)
    
        # 提取"url"的值，也就是pr列表
        for item in data:
            if 'url' in item:

                # 判断pr合入时间是否满足要求
                pr_merge_time = item['merged_at']

                pr_count += 1
                compat_failed = False

                with lock:
                    global_pr_count += 1

                # 0: no exec compat problem
                # 1: have exec compat problem and AI agree with it
                # 2: have exec compat problem and AI disagree with it
                compat_failed_exec = 0

                pr_detail[f'pr{pr_count}'] = {}
                pr_detail[f'pr{pr_count}']['url'] = item['url']
                pr_detail[f'pr{pr_count}']['merge_time'] = pr_merge_time

                # 获取labels的URL
                labels_url = item['url'] + '/labels'
                
                # 执行GET请求获取labels信息
                params = {
                    'access_token': token
                }
                labels_response = requests.get(labels_url, headers=headers, params=params)
                
                if labels_response.status_code == 200:
                    # 解析返回的JSON数据
                    labels_data = json.loads(labels_response.text)
                    pr_detail[f'pr{pr_count}']['label'] = []
                    
                    # 提取"name"的值放进一个列表
                    for label in labels_data:
                        ## 暂时剔除这部分不兼容结果
                        #if 'compat_failed' == label['name']:
                        #    compat_failed += 1
                        #    break
                        
                        if 'mistake/executable' == label['name']:
                            with lock:
                                check_list['compat_exec_failed'] += 1
                            pr_detail[f'pr{pr_count}']['label'].append('compat_exec_failed')
                            if compat_failed_exec != 2:
                                compat_failed_exec = 1

                        if 'mistake/api' == label['name']:
                            with lock:
                                check_list['compat_api_failed'] += 1
                            pr_detail[f'pr{pr_count}']['label'].append('compat_api_failed')
                            compat_failed = True

                        if 'mistake/abi' == label['name']:
                            with lock:
                                check_list['compat_abi_failed'] += 1
                            pr_detail[f'pr{pr_count}']['label'].append('compat_abi_failed')
                            compat_failed = True

                        if 'mistake/ability' == label['name']:
                            with lock:
                                check_list['compat_ability_failed'] += 1
                            pr_detail[f'pr{pr_count}']['label'].append('compat_ability_failed')
                            compat_failed = True

                        if 'mistake/conf' == label['name']:
                            with lock:
                                check_list['compat_conf_failed'] += 1
                            pr_detail[f'pr{pr_count}']['label'].append('compat_conf_failed')
                            compat_failed = True

                        if 'mistake/filelist' == label['name']:
                            with lock:
                                check_list['compat_filelist_failed'] += 1
                            pr_detail[f'pr{pr_count}']['label'].append('compat_filelist_failed')
                            compat_failed = True

                        if 'mistake/subrpmlist' == label['name']:
                            with lock:
                                check_list['compat_subrpm_failed'] += 1
                            pr_detail[f'pr{pr_count}']['label'].append('compat_subrpm_failed')
                            compat_failed = True
                        
                        if 'AI_disagree_exec' == label['name']:
                            with lock:
                                check_list['AI_disagree_exec'] += 1
                            pr_detail[f'pr{pr_count}']['label'].append('AI_disagree_exec')
                            compat_failed_exec = 2


                    # 当前pr是否兼容性检查mistake
                    if compat_failed or compat_failed_exec == 1:
                        compat_failed_count += 1
                else:
                    print(f"Request for labels failed with status code {labels_response.status_code}")

        # 计算每个rpm包中，有兼容性mistake的pr的比例
        #rate = str((compat_failed_count / pr_count) * 100) + "%" if pr_count != 0 else "No pull request."
        #print(f"package: {package},compat_failed rate:{rate}")
    else:
        print(f"Request failed with status code {response.status_code}")

    with lock:
        global_compat_failed += compat_failed_count

    return pr_detail

def dict_to_json_file(my_dict, filename):
    # 使用 json.dump() 将字典写入 JSON 文件
    with open(filename, 'w') as json_file:
        json.dump(my_dict, json_file, indent=4)

def main():
    global global_pr_count, global_compat_failed
    global check_list

    limit_num = 9999999999999
    time_str = "2024-07-29T00:00:00+08:00"

    get_pkg_list(limit_num)

    all_pr_detail = []
    
    # 创建一个线程池
    #with ProcessPoolExecutor(max_workers=10) as executor:
    with ThreadPoolExecutor(max_workers=20) as executor:
        # 提交所有任务
        futures = {executor.submit(get_pr_labels, p, time_str): p for p in all_pkg}

        # 使用 tqdm 包装 as_completed 以显示进度条
        for future in tqdm(as_completed(futures), total=len(futures), desc="Processing packages"):
            pr_detail = future.result()
            all_pr_detail.append(pr_detail)

    rate = str((global_compat_failed / global_pr_count) * 100) + "%" if global_pr_count != 0 else "No pull request."
    print(f"The total compat_failed rate:{rate}")

    for key, value in check_list.items():
        item_rate = str((value / global_pr_count) * 100) + "%" if global_pr_count != 0 else "No pull request."
        print(f"The total {key} rate:{item_rate}")

    print(f"The total pr num:{global_pr_count}")

    dict_to_json_file(all_pr_detail, 'output.json')

if __name__ == "__main__":
    main()
