#!/bin/bash

# 变量
db_name="ocs23.sqlite"
dynamic_library="ELF 64-bit LSB shared object"

# 创建sqlite数据库
sqlite3 "$db_name" "CREATE TABLE so_id (
        so_id INTEGER NOT NULL, 
        pkg_name VARCHAR, 
        so_name VARCHAR, 
        real_so_id INTEGER, 
        PRIMARY KEY (so_id), 
        UNIQUE (so_id)
);"

# 函数，用于插入动态库至数据库并返回创建的so_id
#insert_so() {
#    local pkg_name="$1"
#    local so_name="$2"
#
#    sqlite3 "$db_name" "INSERT INTO so_id (pkg_name, so_name) VALUES ('$pkg_name', '$so_name');"
#    # 获取刚插入的so_id
#    sqlite3 "$db_name" "SELECT last_insert_rowid();"
#}

insert_so() {
    local pkg_name="$1"
    local so_name="$2"

    sqlite3 "$db_name" "BEGIN;
                       INSERT INTO so_id (pkg_name, so_name) VALUES ('$pkg_name', '$so_name');
                       SELECT last_insert_rowid();
                       COMMIT;"
}

# 从所有安装的rpm包中搜索动态库
rpm -qa | while read pkg_name; do
    rpm -ql "$pkg_name" | while read file; do
        if echo "$file" | grep "/usr/lib/.build-id/" > /dev/null; then
            continue
        fi
        file_type=$(file -b "$file")
        if echo "$file_type" | grep -w "$dynamic_library" > /dev/null; then
            # 插入动态库至数据库并获取创建的so_id
            so_id=$(insert_so "$pkg_name" "$file")
            # 将real_so_id设置为so_id
            sqlite3 "$db_name" "UPDATE so_id SET real_so_id = '$so_id' WHERE so_id = '$so_id';"
        elif [ -L "$file" ]; then
            # 检查文件是否为指向共享库的符号链接
            target_file=$(readlink -f "$file")
            target_file_type=$(file -b "$target_file")
            if echo "$target_file_type" | grep -w "$dynamic_library" > /dev/null; then
                # 仅插入符号链接
                sqlite3 "$db_name" "INSERT INTO so_id (pkg_name, so_name, real_so_id) VALUES ('$pkg_name', '$file', '');"
            fi
        fi
    done
done

# 从数据库中获取所有记录
all_rows=$(sqlite3 "$db_name" "SELECT so_id, so_name FROM so_id;")

# 遍历所有记录
while IFS='|' read -ra row; do
    so_id="${row[0]}"
    so_name="${row[1]}"

    # 如果是符号链接
    if [ -L "$so_name" ]; then
        target_file=$(readlink -f "$so_name")  # 解析目标文件

        # 检查目标文件是否是动态库
        file_type=$(file -b "$target_file")
        if echo "$file_type" | grep -w "$dynamic_library" > /dev/null; then
            # 从数据库中获取目标文件的so_id
            real_so_id=$(sqlite3 "$db_name" "SELECT so_id FROM so_id WHERE so_name='$target_file';")

            # 更新当前记录的real_so_id列
            sqlite3 "$db_name" "UPDATE so_id SET real_so_id='$real_so_id' WHERE so_id='$so_id';"
        fi
    fi
done <<< "$all_rows"
